module.exports={A:{A:{"1":"D A B","2":"I F E hB"},B:{"1":"C O P H J K L","16":"S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P gB oB"},D:{"1":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k","2":"0 1 2 3 4 5 6 7 8 9 l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB","16":"pB rB YB"},E:{"1":"I F E D A B C O P bB cB dB eB TB U x iB jB","2":"G V ZB SB"},F:{"1":"B C H J K L W X Y Z a b lB mB nB U VB PC x","2":"0 1 2 3 4 5 6 7 8 9 D c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB"},G:{"1":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB qB XB"},H:{"16":"9B"},I:{"1":"G M DC XB EC FC","16":"OB AC BC CC"},J:{"16":"F A"},K:{"1":"C Q x","16":"A B U VB"},L:{"1":"M"},M:{"1":"N"},N:{"16":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"2":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:1,C:"Media attribute"};
