module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S UB M HB","2":"C O P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB","164":"OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r gB oB"},D:{"1":"IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","292":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N"},E:{"1":"O P x iB jB","292":"G V I F E D A B C ZB SB bB cB dB eB TB U"},F:{"1":"T R N","2":"D B C kB lB mB nB U VB PC x","292":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB"},G:{"1":"3B 4B 5B 6B 7B 8B","292":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B"},H:{"2":"9B"},I:{"1":"M","292":"OB G AC BC CC DC XB EC FC"},J:{"292":"F A"},K:{"2":"A B C U VB x","292":"Q"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"292":"GC"},P:{"1":"TB MC","292":"G HC IC JC KC LC"},Q:{"292":"NC"},R:{"292":"OC"},S:{"1":"fB"}},B:5,C:"CSS Logical Properties"};
