module.exports={A:{A:{"1":"B","2":"I F E D hB","289":"A"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f gB oB","194":"0 1 2 g h i j k l m n o p q r s t u v w Q y z","1025":"3 4 5 6 7"},D:{"1":"0 1 2 3 4 5 6 7 8 9 n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m"},E:{"2":"G V I F E D A B C O P ZB SB bB cB dB eB TB U x iB jB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C H J K L W X Y Z kB lB mB nB U VB PC x"},G:{"1":"4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB wB","516":"xB yB zB 0B 1B 2B 3B"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"1":"B","289":"A"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"194":"fB"}},B:2,C:"CSS touch-action property"};
