module.exports={A:{A:{"2":"I F E D hB","129":"A B"},B:{"1":"S UB M HB","129":"C O","1025":"P H J K L"},C:{"2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f gB oB","513":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"1":"0 1 2 3 4 5 6 7 8 9 I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V"},E:{"1":"V I F E D A B C O P bB cB dB eB TB U x iB jB","2":"G ZB SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB lB mB nB U VB PC x"},G:{"388":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB AC BC CC","388":"G M DC XB EC FC"},J:{"2":"F","388":"A"},K:{"1":"A B C U VB x","388":"Q"},L:{"388":"M"},M:{"641":"N"},N:{"388":"A B"},O:{"388":"GC"},P:{"388":"G HC IC JC KC LC TB MC"},Q:{"388":"NC"},R:{"388":"OC"},S:{"513":"fB"}},B:1,C:"Number input type"};
