module.exports={A:{A:{"2":"I F E hB","260":"D A B"},B:{"1":"S UB M HB","260":"C O P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z gB oB","132":"a b c d e f"},D:{"1":"2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"0 1 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},E:{"1":"B C O P TB U x iB jB","2":"G V I F E D A ZB SB bB cB dB eB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 p q r s t u v w Q y z BB DB EB CB AB T R N x","2":"D B H J K L W X Y Z a b c d e f g h i j k l m n o kB lB mB nB U VB PC","16":"C"},G:{"1":"zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB wB xB yB"},H:{"1":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"1":"x","2":"A B U VB","16":"C Q"},L:{"1":"M"},M:{"1":"N"},N:{"260":"A B"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","2":"G"},Q:{"2":"NC"},R:{"2":"OC"},S:{"1":"fB"}},B:5,C:"KeyboardEvent.key"};
