module.exports={A:{A:{"1":"B","2":"I F E D A hB"},B:{"2":"C O P H J K L S UB M HB"},C:{"1":"0 1 O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","2":"2 3 4 5 6 7 8 9 aB OB G V I F E D A B C WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB"},D:{"1":"0 1 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","2":"2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"E D A B C eB TB U","2":"G V I F ZB SB bB cB dB","129":"O P x iB jB"},F:{"1":"H J K L W X Y Z a b c d e f g h i j k l m n o p q t v x","2":"0 1 2 3 4 5 6 7 8 9 D B C r s u w Q y z BB DB EB CB AB T R N kB lB mB nB U VB PC"},G:{"1":"E vB wB xB yB zB 0B 1B 2B","2":"SB qB XB sB tB uB","257":"3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"OB G DC XB EC FC","2":"M AC BC CC"},J:{"2":"F A"},K:{"1":"x","2":"A B C Q U VB"},L:{"2":"M"},M:{"2":"N"},N:{"1":"B","2":"A"},O:{"2":"GC"},P:{"1":"G","2":"HC IC JC KC LC TB MC"},Q:{"2":"NC"},R:{"16":"OC"},S:{"1":"fB"}},B:7,C:"SPDY protocol"};
