module.exports={A:{A:{"2":"D A B hB","4":"I F E"},B:{"2":"C O P H J K L","8":"S UB M HB"},C:{"8":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB"},D:{"8":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"8":"G V I F E D A B C O P ZB SB bB cB dB eB TB U x iB jB"},F:{"8":"0 1 2 3 4 5 6 7 8 9 D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB lB mB nB U VB PC x"},G:{"8":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"8":"9B"},I:{"8":"OB G M AC BC CC DC XB EC FC"},J:{"8":"F A"},K:{"8":"A B C Q U VB x"},L:{"8":"M"},M:{"8":"N"},N:{"2":"A B"},O:{"8":"GC"},P:{"8":"G HC IC JC KC LC TB MC"},Q:{"8":"NC"},R:{"8":"OC"},S:{"8":"fB"}},B:7,C:"XHTML+SMIL animation"};
