module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P H J K L","164":"S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v gB oB","322":"w"},D:{"2":"G V I F E D A B C O P H J K L W X Y Z a b","164":"0 1 2 3 4 5 6 7 8 9 c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"E D A B C O P dB eB TB U x iB jB","2":"G V I ZB SB bB","164":"F cB"},F:{"2":"D B C kB lB mB nB U VB PC x","164":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"1":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB sB tB"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB","164":"M EC FC"},J:{"2":"F","164":"A"},K:{"2":"A B C U VB x","164":"Q"},L:{"164":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"164":"GC"},P:{"164":"G HC IC JC KC LC TB MC"},Q:{"164":"NC"},R:{"164":"OC"},S:{"1":"fB"}},B:4,C:"text-emphasis styling"};
