module.exports={A:{A:{"2":"I F E D hB","132":"A B"},B:{"132":"C O P H J K L","1028":"S UB M HB"},C:{"2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j gB oB","2564":"k l m n o p q r s t u v w Q y z","3076":"0 1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"16":"G V I F","132":"0 1 2 3 4 5 6 7 8 9 D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB","388":"E","1028":"GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"16":"G ZB SB","132":"V I F E D A bB cB dB eB TB","1028":"B C O P U x iB jB"},F:{"2":"D B C kB lB mB nB U VB PC x","132":"H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y","1028":"0 1 2 3 4 5 6 7 8 9 z BB DB EB CB AB T R N"},G:{"16":"SB qB XB","132":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"M","132":"G DC XB EC FC","292":"OB AC BC CC"},J:{"16":"F","132":"A"},K:{"2":"A B C U VB x","132":"Q"},L:{"1028":"M"},M:{"1":"N"},N:{"132":"A B"},O:{"132":"GC"},P:{"132":"G HC IC JC KC LC TB MC"},Q:{"132":"NC"},R:{"132":"OC"},S:{"2564":"fB"}},B:4,C:"DOMMatrix"};
