module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"H J K L S UB M HB","2":"C O P"},C:{"1":"0 1 2 3 4 5 6 7 8 9 e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","33":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d gB oB"},D:{"1":"N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","33":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R"},E:{"1":"B C O P U x iB jB","33":"G V I F E D A ZB SB bB cB dB eB TB"},F:{"1":"6 7 8 9 C BB DB EB CB AB T R N PC x","2":"D B kB lB mB nB U VB","33":"0 1 2 3 4 5 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"33":"F A"},K:{"2":"A B C U VB x","33":"Q"},L:{"1":"M"},M:{"2":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"2":"G HC IC JC KC LC TB MC"},Q:{"33":"NC"},R:{"2":"OC"},S:{"2":"fB"}},B:3,C:"CSS grab & grabbing cursors"};
