module.exports={A:{A:{"1":"D A B","2":"I F E hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB","36":"oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","516":"G V I F E D A B C O P"},E:{"1":"F E D A B C O P dB eB TB U x iB jB","772":"G V I ZB SB bB cB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N mB nB U VB PC x","2":"D kB","36":"lB"},G:{"1":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","4":"SB qB XB tB","516":"sB"},H:{"132":"9B"},I:{"1":"M EC FC","36":"AC","516":"OB G DC XB","548":"BC CC"},J:{"1":"F A"},K:{"1":"A B C Q U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"1":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:4,C:"CSS3 Background-image options"};
