module.exports={A:{A:{"2":"I F E A B hB","16":"D"},B:{"2":"C O P H J K L S UB M HB"},C:{"2":"0 1 2 3 4 5 6 7 8 9 aB OB I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB","16":"G V"},D:{"2":"0 1 2 3 4 5 6 7 8 9 G V I F E D A O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","16":"B C"},E:{"2":"G I ZB SB bB","16":"V F E D A B C O P cB dB eB TB U x iB jB"},F:{"2":"0 1 2 3 4 5 6 7 8 9 D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB lB mB nB VB PC x","16":"U"},G:{"2":"SB qB XB sB tB","16":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G M AC BC DC XB EC FC","16":"CC"},J:{"2":"A","16":"F"},K:{"2":"A B C Q U VB x"},L:{"2":"M"},M:{"2":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"2":"G HC IC JC KC LC TB MC"},Q:{"2":"NC"},R:{"2":"OC"},S:{"2":"fB"}},B:7,C:"Test feature - updated"};
