module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S UB M HB","2":"C","260":"O P H J K L"},C:{"1":"1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f gB oB","516":"0 g h i j k l m n o p q r s t u v w Q y z"},D:{"1":"4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G","16":"V I F E D A B C O P","260":"3","772":"0 1 2 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},E:{"1":"B C O P TB U x iB jB","2":"G ZB SB","16":"V","772":"I F E D A bB cB dB eB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 r s t u v w Q y z BB DB EB CB AB T R N","16":"D kB","260":"B C q lB mB nB U VB PC x","772":"H J K L W X Y Z a b c d e f g h i j k l m n o p"},G:{"1":"zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB","772":"E sB tB uB vB wB xB yB"},H:{"132":"9B"},I:{"1":"M","2":"OB AC BC CC","260":"G DC XB EC FC"},J:{"2":"F","260":"A"},K:{"1":"Q","260":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","260":"G"},Q:{"1":"NC"},R:{"1":"OC"},S:{"516":"fB"}},B:5,C:":in-range and :out-of-range CSS pseudo-classes"};
