module.exports={A:{A:{"2":"hB","8":"I F E","129":"A B","161":"D"},B:{"1":"K L S UB M HB","129":"C O P H J"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB","33":"G V I F E D A B C O P H gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","33":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m"},E:{"1":"D A B C O P eB TB U x iB jB","33":"G V I F E ZB SB bB cB dB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N x","2":"D kB lB","33":"B C H J K L W X Y Z mB nB U VB PC"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","33":"E SB qB XB sB tB uB vB"},H:{"2":"9B"},I:{"1":"M","33":"OB G AC BC CC DC XB EC FC"},J:{"33":"F A"},K:{"1":"B C Q U VB x","2":"A"},L:{"1":"M"},M:{"1":"N"},N:{"1":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:5,C:"CSS3 2D Transforms"};
