module.exports={A:{A:{"2":"hB","8":"I F E","129":"D A B"},B:{"1":"K L S UB M HB","129":"C O P H J"},C:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","8":"aB OB gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","8":"G V I"},E:{"1":"D A B C O P eB TB U x iB jB","8":"G V ZB SB","129":"I F E bB cB dB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N PC x","2":"B nB U VB","8":"D kB lB mB"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","8":"SB qB XB","129":"E sB tB uB vB"},H:{"1":"9B"},I:{"1":"M EC FC","2":"AC BC CC","129":"OB G DC XB"},J:{"1":"A","129":"F"},K:{"1":"C Q x","8":"A B U VB"},L:{"1":"M"},M:{"1":"N"},N:{"129":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:1,C:"Inline SVG in HTML5"};
