module.exports={A:{A:{"2":"hB","4":"A B","8":"I F E D"},B:{"1":"J K L S UB M HB","4":"C O P H"},C:{"4":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","8":"aB OB gB oB"},D:{"1":"GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","4":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB"},E:{"4":"G V I F E D A B C O P bB cB dB eB TB U x iB jB","8":"ZB SB"},F:{"1":"3 4 5 6 7 8 9 D B C BB DB EB CB AB T R N kB lB mB nB U VB PC x","4":"0 1 2 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},G:{"2":"SB","4":"E qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB","4":"EC FC"},J:{"2":"F","4":"A"},K:{"1":"A B C U VB x","4":"Q"},L:{"1":"M"},M:{"4":"N"},N:{"4":"A B"},O:{"1":"GC"},P:{"1":"KC LC TB MC","4":"G HC IC JC"},Q:{"1":"NC"},R:{"4":"OC"},S:{"4":"fB"}},B:1,C:"HTML5 form features"};
