module.exports={A:{A:{"2":"I F E hB","132":"D A B"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB","260":"G V I F E D A B C O P H gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","548":"G V I F E D A B C O P H J K L W X Y Z a b c d e f"},E:{"2":"ZB SB","548":"G V I F E D A B C O P bB cB dB eB TB U x iB jB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N x","2":"D","548":"B C kB lB mB nB U VB PC"},G:{"16":"SB","548":"E qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"132":"9B"},I:{"1":"M EC FC","16":"AC BC","548":"OB G CC DC XB"},J:{"548":"F A"},K:{"1":"Q x","548":"A B C U VB"},L:{"1":"M"},M:{"1":"N"},N:{"132":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:2,C:"Media Queries: resolution feature"};
