module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"O P H J K L S UB M HB","2":"C"},C:{"1":"0 1 2 3 4 5 6 7 8 9 s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O gB oB","33":"P H J K L W X Y Z a b c d e f g h i j k l m n o p q r"},D:{"1":"0 1 2 3 4 5 6 7 8 9 o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H","33":"Z a b c d e f g h i j k l m n","66":"J K L W X Y"},E:{"1":"B C O P TB U x iB jB","2":"G V I F E D A ZB SB bB cB dB eB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","33":"H J K L W X Y Z a"},G:{"1":"zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB wB xB yB"},H:{"2":"9B"},I:{"2":"OB G M AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C Q U VB x"},L:{"2":"M"},M:{"2":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"2":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"2":"OC"},S:{"1":"fB"}},B:2,C:"Pointer Lock API"};
