module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"K L S UB M HB","2":"C O P H J"},C:{"2":"0 1 2 3 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z gB oB","4609":"DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","4674":"GB","5698":"BB","7490":"4 5 6 7 8","7746":"9 WB"},D:{"1":"R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","4097":"T","4290":"WB BB GB","6148":"DB EB CB AB"},E:{"2":"G V I F E D A ZB SB bB cB dB eB TB","4609":"B C U x","8193":"O P iB jB"},F:{"1":"5 6 7 8 9 BB DB EB CB AB T R N","2":"D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z kB lB mB nB U VB PC x","4097":"4","6148":"0 1 2 3"},G:{"1":"4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB wB xB yB zB","4097":"0B 1B 2B 3B"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C Q U VB x"},L:{"1":"M"},M:{"4097":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"2":"G HC IC JC","4097":"KC LC TB MC"},Q:{"4097":"NC"},R:{"2":"OC"},S:{"2":"fB"}},B:5,C:"Variable fonts"};
