module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"K L","2":"C O P H J","257":"S UB M HB"},C:{"2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u gB oB","257":"0 1 2 4 5 6 7 8 9 v Q y z WB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","1281":"3 w BB"},D:{"2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u","257":"1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","388":"0 v w Q y z"},E:{"2":"G V I F E D ZB SB bB cB dB","514":"A B C O P eB TB U x iB jB"},F:{"2":"D B C H J K L W X Y Z a b c d e f g h i j k l m n kB lB mB nB U VB PC x","16":"o p q r s","257":"0 1 2 3 4 5 6 7 8 9 t u v w Q y z BB DB EB CB AB T R N"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G M AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"2":"OC"},S:{"257":"fB"}},B:5,C:"Push API"};
