module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S UB M HB","2":"C O","322":"P","8196":"H J K L"},C:{"2":"0 1 2 3 4 5 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z gB oB","4162":"6 7 8 9 WB BB GB DB EB CB AB","16452":"T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"1":"RB S UB M HB pB rB YB","2":"0 1 2 3 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","194":"4 5 6 7 8 9","1090":"WB BB","8196":"GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB"},E:{"1":"O P x iB jB","2":"G V I F E D ZB SB bB cB dB eB","514":"A B TB","8196":"C U"},F:{"1":"T R N","2":"D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q kB lB mB nB U VB PC x","194":"r s t u v w Q y","8196":"0 1 2 3 4 5 6 7 8 9 z BB DB EB CB AB"},G:{"1":"3B 4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB wB xB","514":"yB zB 0B","8196":"1B 2B"},H:{"2":"9B"},I:{"2":"OB G M AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C Q U VB x"},L:{"2052":"M"},M:{"2":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"2":"G","8196":"HC IC JC KC LC TB MC"},Q:{"8196":"NC"},R:{"2":"OC"},S:{"2":"fB"}},B:4,C:"Payment Request API"};
