module.exports={A:{A:{"2":"I hB","2340":"F E D A B"},B:{"2":"C O P H J K L","1025":"S UB M HB"},C:{"2":"aB OB gB","513":"AB T R N IB JB KB LB MB NB FB PB QB RB S","545":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB oB"},D:{"2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r","1025":"0 1 2 3 4 5 6 7 8 9 s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"A B C O P TB U x iB jB","2":"G V ZB SB bB","164":"I","4644":"F E D cB dB eB"},F:{"2":"D B H J K L W X Y Z a b c d e kB lB mB nB U VB","545":"C PC x","1025":"0 1 2 3 4 5 6 7 8 9 f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"1":"yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB","4260":"sB tB","4644":"E uB vB wB xB"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB EC FC","1025":"M"},J:{"2":"F","4260":"A"},K:{"2":"A B U VB","545":"C x","1025":"Q"},L:{"1025":"M"},M:{"545":"N"},N:{"2340":"A B"},O:{"1":"GC"},P:{"1025":"G HC IC JC KC LC TB MC"},Q:{"1025":"NC"},R:{"1025":"OC"},S:{"4097":"fB"}},B:7,C:"Crisp edges/pixelated images"};
