module.exports={A:{A:{"2":"hB","8":"I F E","772":"D A B"},B:{"1":"S UB M HB","513":"C O P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB","4":"aB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"G V I F E D A B C O P SB bB cB dB eB TB U x iB jB","4":"ZB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB lB mB nB U VB PC x"},G:{"1":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"1":"9B"},I:{"1":"M EC FC","2":"AC BC CC","132":"OB G DC XB"},J:{"1":"F A"},K:{"1":"A B C Q U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"257":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:4,C:"SVG (basic support)"};
