module.exports={A:{A:{"2":"I F E D A hB","2052":"B"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","194":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L","322":"W X Y Z a b c d e f g h i j k l m n o p q r","516":"s t u v w Q y z"},E:{"1":"B C O P U x iB jB","2":"G V I F E D ZB SB bB cB dB eB","1028":"A TB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","322":"H J K L W X Y Z a b c d e","516":"f g h i j k l m"},G:{"1":"0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB wB xB","1028":"yB zB"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"1":"B","2":"A"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","516":"G"},Q:{"1":"NC"},R:{"516":"OC"},S:{"1":"fB"}},B:6,C:"let"};
