module.exports={A:{A:{"1":"D A B","16":"hB","260":"I F E"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","132":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t gB oB","2180":"0 1 2 u v w Q y z"},D:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","16":"G V I F E D A B C O P"},E:{"1":"I F E D A B C O P bB cB dB eB TB U x iB jB","16":"G V ZB SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","132":"D B C kB lB mB nB U VB PC x"},G:{"16":"XB","132":"SB qB","516":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"M EC FC","16":"OB G AC BC CC DC","1025":"XB"},J:{"1":"A","16":"F"},K:{"1":"Q","16":"A B C U VB","132":"x"},L:{"1":"M"},M:{"1":"N"},N:{"1":"B","16":"A"},O:{"1025":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"2180":"fB"}},B:5,C:"Selection API"};
