module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"O P H J K L S UB M HB","2":"C"},C:{"1":"RB S","16":"aB","33":"0 1 2 3 4 5 6 7 8 9 OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","16":"G V I F E D A B C O P","132":"H J K L W X Y Z a b c d e f g h i j k l m"},E:{"1":"D A B C O P eB TB U x iB jB","16":"ZB SB","132":"G V I F E bB cB dB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","16":"D B kB lB mB nB U","132":"C H J K L W X Y Z VB PC x"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB qB","132":"E XB sB tB uB vB"},H:{"2":"9B"},I:{"1":"M","16":"AC BC","132":"OB G CC DC XB EC FC"},J:{"1":"A","132":"F"},K:{"1":"Q","2":"A B U","132":"C VB x"},L:{"1":"M"},M:{"33":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"33":"fB"}},B:1,C:"CSS :read-only and :read-write selectors"};
