module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P H J K","260":"S UB M HB","3138":"L"},C:{"1":"5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB","132":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q gB oB","644":"0 1 2 3 4 y z"},D:{"2":"G V I F E D A B C O P H J K L W X Y Z a","260":"6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","292":"0 1 2 3 4 5 b c d e f g h i j k l m n o p q r s t u v w Q y z"},E:{"2":"G V I ZB SB bB cB","292":"F E D A B C O P dB eB TB U x iB jB"},F:{"2":"D B C kB lB mB nB U VB PC x","260":"0 1 2 3 4 5 6 7 8 9 t u v w Q y z BB DB EB CB AB T R N","292":"H J K L W X Y Z a b c d e f g h i j k l m n o p q r s"},G:{"2":"SB qB XB sB tB","292":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB","260":"M","292":"EC FC"},J:{"2":"F A"},K:{"2":"A B C U VB x","292":"Q"},L:{"260":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"292":"GC"},P:{"292":"G HC IC JC KC LC TB MC"},Q:{"292":"NC"},R:{"260":"OC"},S:{"644":"fB"}},B:4,C:"CSS clip-path property (for HTML)"};
