module.exports={A:{A:{"2":"I F E hB","8":"D","292":"A B"},B:{"1":"J K L S UB M HB","292":"C O P H"},C:{"1":"5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L gB oB","8":"W X Y Z a b c d e f g h i j k l m n o p q","584":"0 1 2 r s t u v w Q y z","1025":"3 4"},D:{"1":"9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L W X Y Z a b","8":"c d e f","200":"0 1 2 3 4 5 6 7 g h i j k l m n o p q r s t u v w Q y z","1025":"8"},E:{"1":"B C O P TB U x iB jB","2":"G V ZB SB bB","8":"I F E D A cB dB eB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 v w Q y z BB DB EB CB AB T R N","2":"D B C H J K L W X Y Z a b c d e kB lB mB nB U VB PC x","200":"f g h i j k l m n o p q r s t u"},G:{"1":"zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB sB","8":"E tB uB vB wB xB yB"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC","8":"XB EC FC"},J:{"2":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"292":"A B"},O:{"1":"GC"},P:{"1":"IC JC KC LC TB MC","2":"HC","8":"G"},Q:{"1":"NC"},R:{"2":"OC"},S:{"1":"fB"}},B:4,C:"CSS Grid Layout (level 1)"};
