module.exports={A:{A:{"2":"D A B hB","8":"I F E"},B:{"2":"C O P H J K L","8":"S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","129":"aB OB gB oB"},D:{"1":"b","8":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"A B C O P TB U x iB jB","260":"G V I F E D ZB SB bB cB dB eB"},F:{"2":"D","4":"B C kB lB mB nB U VB PC x","8":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"1":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","8":"SB qB XB"},H:{"8":"9B"},I:{"8":"OB G M AC BC CC DC XB EC FC"},J:{"1":"A","8":"F"},K:{"8":"A B C Q U VB x"},L:{"8":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"4":"GC"},P:{"8":"G HC IC JC KC LC TB MC"},Q:{"8":"NC"},R:{"8":"OC"},S:{"1":"fB"}},B:2,C:"MathML"};
