module.exports={A:{A:{"1":"B","2":"I F E D hB","164":"A"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V gB oB","8":"I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r","328":"0 1 2 3 4 5 6 7 8 9 s t u v w Q y z"},D:{"1":"6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L W X Y","8":"0 1 2 Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","584":"3 4 5"},E:{"1":"O P iB jB","2":"G V I ZB SB bB","8":"F E D A B C cB dB eB TB U","1096":"x"},F:{"1":"0 1 2 3 4 5 6 7 8 9 t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","8":"H J K L W X Y Z a b c d e f g h i j k l m n o p","584":"q r s"},G:{"1":"5B 6B 7B 8B","8":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B","6148":"4B"},H:{"2":"9B"},I:{"1":"M","8":"OB G AC BC CC DC XB EC FC"},J:{"8":"F A"},K:{"1":"Q","2":"A","8":"B C U VB x"},L:{"1":"M"},M:{"328":"N"},N:{"1":"B","36":"A"},O:{"8":"GC"},P:{"1":"IC JC KC LC TB MC","2":"HC","8":"G"},Q:{"1":"NC"},R:{"2":"OC"},S:{"328":"fB"}},B:2,C:"Pointer events"};
