module.exports={A:{A:{"1":"A B","2":"I F E D hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G"},E:{"1":"V I F E D A B C O P bB cB dB eB TB U x iB jB","2":"G ZB SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB lB mB nB U VB PC x","2":"D"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"OB G M DC XB EC FC","2":"AC BC CC"},J:{"1":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"1":"A B"},O:{"2":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"2":"fB"}},B:1,C:"Autofocus attribute"};
