module.exports={A:{A:{"388":"A B","900":"I F E D hB"},B:{"388":"C O P H J K L","900":"S UB M HB"},C:{"772":"AB T R N IB JB KB LB MB NB FB PB QB RB S","900":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB gB oB"},D:{"900":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"772":"A","900":"G V I F E D B C O P ZB SB bB cB dB eB TB U x iB jB"},F:{"16":"D kB","129":"B C lB mB nB U VB PC x","900":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"900":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"129":"9B"},I:{"900":"OB G M AC BC CC DC XB EC FC"},J:{"900":"F A"},K:{"129":"A B C U VB x","900":"Q"},L:{"900":"M"},M:{"900":"N"},N:{"388":"A B"},O:{"900":"GC"},P:{"900":"G HC IC JC KC LC TB MC"},Q:{"900":"NC"},R:{"900":"OC"},S:{"900":"fB"}},B:2,C:"CSS page-break properties"};
