module.exports={A:{A:{"1":"A B","2":"I F E D hB"},B:{"1":"C O P H J K L","516":"S UB M HB"},C:{"132":"3 4 5 6 7 8 9 WB BB GB DB EB CB","164":"0 1 2 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z gB oB","516":"AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"420":"0 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","516":"1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"A B C O P TB U x iB jB","132":"D eB","164":"F E dB","420":"G V I ZB SB bB cB"},F:{"1":"C U VB PC x","2":"D B kB lB mB nB","420":"H J K L W X Y Z a b c d e f g h i j k l m n","516":"0 1 2 3 4 5 6 7 8 9 o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"1":"yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","132":"wB xB","164":"E uB vB","420":"SB qB XB sB tB"},H:{"1":"9B"},I:{"420":"OB G AC BC CC DC XB EC FC","516":"M"},J:{"420":"F A"},K:{"1":"C U VB x","2":"A B","132":"Q"},L:{"516":"M"},M:{"132":"N"},N:{"1":"A B"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","420":"G"},Q:{"132":"NC"},R:{"132":"OC"},S:{"164":"fB"}},B:4,C:"CSS3 Multiple column layout"};
