module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S UB M HB","36":"C O P H J K L"},C:{"1":"2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L gB oB","33":"0 1 W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},D:{"1":"8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","36":"0 1 2 3 4 5 6 7 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},E:{"1":"B C O P TB U x iB jB","2":"G ZB SB","36":"V I F E D A bB cB dB eB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","36":"H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u"},G:{"1":"zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB","36":"E XB sB tB uB vB wB xB yB"},H:{"2":"9B"},I:{"1":"M","36":"OB G AC BC CC DC XB EC FC"},J:{"36":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"36":"A B"},O:{"1":"GC"},P:{"1":"JC KC LC TB MC","36":"G HC IC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"33":"fB"}},B:5,C:"::placeholder CSS pseudo-element"};
