module.exports={A:{A:{"2":"D A B hB","8":"I F E"},B:{"1":"S UB M HB","2":"C O P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB","8":"OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q gB oB","194":"y z"},D:{"1":"0 1 2 3 4 5 6 7 8 9 n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","8":"G V I F E D A B","257":"W X Y Z a b c d e f g h i j k l m","769":"C O P H J K L"},E:{"1":"C O P x iB jB","8":"G V ZB SB bB","257":"I F E D A cB dB eB","1025":"B TB U"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"C U VB PC x","8":"D B kB lB mB nB"},G:{"1":"E tB uB vB wB xB 1B 2B 3B 4B 5B 6B 7B 8B","8":"SB qB XB sB","1025":"yB zB 0B"},H:{"8":"9B"},I:{"1":"G M DC XB EC FC","8":"OB AC BC CC"},J:{"1":"A","8":"F"},K:{"1":"Q","8":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"769":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:1,C:"Details & Summary elements"};
