module.exports={A:{A:{"2":"K C G E A B CB"},B:{"2":"D v Z I M H"},C:{"2":"VB y F J K C G E A B D v Z I M H N O P Q R S T U V W X Y x a b c d e f L h i j k l m n o p q r s t u z TB SB","129":"0 1 2 3 5 6 7 w"},D:{"1":"2 3 5 6 7 w HB g DB XB EB FB","2":"0 1 F J K C G E A B D v Z I M H N O P Q R S T U V W X Y x a b c d e f L h i j k l m n o p q r s t u z"},E:{"2":"F J K C G E A B GB AB IB JB KB LB MB NB"},F:{"1":"l m n o p q r s t u","2":"8 9 E B D I M H N O P Q R S T U V W X Y x a b c d e f L h i j k OB PB QB RB UB BB"},G:{"2":"4 G AB WB YB ZB aB bB cB dB eB fB gB"},H:{"2":"hB"},I:{"1":"g","2":"4 y F iB jB kB lB mB nB"},J:{"2":"C","16":"A"},K:{"2":"8 9 A B D L BB"},L:{"1":"g"},M:{"2":"w"},N:{"2":"A B"},O:{"16":"oB"},P:{"1":"J pB","16":"F"},Q:{"16":"qB"},R:{"1":"rB"}},B:5,C:"Auxclick"};
