module.exports={A:{A:{"1":"A B","2":"I F E D hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S oB","2":"aB OB gB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G"},E:{"1":"G V I F E D A B C O P bB cB dB eB TB U x iB jB","2":"ZB SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N nB U VB PC x","2":"D kB lB mB"},G:{"1":"E tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB sB"},H:{"130":"9B"},I:{"130":"OB G M AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"130":"A B C Q U VB x"},L:{"132":"M"},M:{"130":"N"},N:{"2":"A B"},O:{"130":"GC"},P:{"130":"G","132":"HC IC JC KC LC TB MC"},Q:{"132":"NC"},R:{"132":"OC"},S:{"2":"fB"}},B:1,C:"Multiple file selection"};
