module.exports={A:{A:{"388":"I F E D A B hB"},B:{"260":"S UB M HB","388":"C O P H J K L"},C:{"260":"GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","388":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB gB oB"},D:{"260":"N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","388":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R"},E:{"260":"P iB jB","388":"G V I F E D A B C O ZB SB bB cB dB eB TB U x"},F:{"260":"6 7 8 9 BB DB EB CB AB T R N","388":"0 1 2 3 4 5 D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z kB lB mB nB U VB PC x"},G:{"260":"7B 8B","388":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B"},H:{"388":"9B"},I:{"388":"OB G M AC BC CC DC XB EC FC"},J:{"388":"F A"},K:{"388":"A B C Q U VB x"},L:{"260":"M"},M:{"260":"N"},N:{"388":"A B"},O:{"388":"GC"},P:{"388":"G HC IC JC KC LC TB MC"},Q:{"388":"NC"},R:{"388":"OC"},S:{"388":"fB"}},B:5,C:"CSS overflow property"};
