"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "default", {
  enumerable: true,
  get: function () {
    return _regeneratorTransform.default;
  }
});

var _regeneratorTransform = _interopRequireDefault(require("regenerator-transform"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }