module.exports={A:{A:{"1":"A B","2":"I F E D hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E"},E:{"1":"I F E D A B C O P bB cB dB eB TB U x iB jB","2":"G V ZB SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N mB nB U VB PC x","2":"D kB lB"},G:{"4":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"4":"9B"},I:{"4":"OB G M AC BC CC DC XB EC FC"},J:{"1":"A","4":"F"},K:{"4":"A B C Q U VB x"},L:{"4":"M"},M:{"4":"N"},N:{"4":"A B"},O:{"4":"GC"},P:{"4":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"4":"OC"},S:{"2":"fB"}},B:1,C:"Spellcheck attribute"};
