module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S UB M HB","2":"C O","132":"P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h gB oB","132":"i j k l m n o p q r s t u v w Q y"},D:{"1":"N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m","132":"0 1 2 3 4 5 6 7 8 9 n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R"},E:{"1":"A B C O P eB TB U x iB jB","2":"G V I F ZB SB bB cB","132":"E D dB"},F:{"1":"6 7 8 9 BB DB EB CB AB T R N","2":"D B C H J K L W X Y Z kB lB mB nB U VB PC x","132":"0 1 2 3 4 5 a b c d e f g h i j k l m n o p q r s t u v w Q y z"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB sB tB uB","16":"E","132":"vB"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"1":"A","2":"F"},K:{"2":"A B C U VB x","132":"Q"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"132":"GC"},P:{"1":"TB MC","132":"G HC IC JC KC LC"},Q:{"132":"NC"},R:{"132":"OC"},S:{"1":"fB"}},B:1,C:"Path2D"};
