module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P H J K L","1537":"S UB M HB"},C:{"2":"aB","932":"0 1 2 3 4 5 6 7 8 9 OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB gB oB","2308":"T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"2":"G V I F E D A B C O P H J K L W X Y","545":"Z a b c d e f g h i j k l m n o p q r s t u v w","1537":"0 1 2 3 4 5 6 7 8 9 Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G V I ZB SB bB","516":"B C O P U x iB jB","548":"D A eB TB","676":"F E cB dB"},F:{"2":"D B C kB lB mB nB U VB PC x","513":"l","545":"H J K L W X Y Z a b c d e f g h i j","1537":"0 1 2 3 4 5 6 7 8 9 k m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"2":"SB qB XB sB tB","548":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","676":"E uB vB"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB","545":"EC FC","1537":"M"},J:{"2":"F","545":"A"},K:{"2":"A B C U VB x","1537":"Q"},L:{"1537":"M"},M:{"2340":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"545":"G","1537":"HC IC JC KC LC TB MC"},Q:{"545":"NC"},R:{"1537":"OC"},S:{"932":"fB"}},B:5,C:"Intrinsic & Extrinsic Sizing"};
