module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P H J K L","1348":"S UB M HB"},C:{"16":"aB OB gB oB","548":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"16":"G V I F E D A B C O P","164":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB","1348":"AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G ZB SB","16":"V","164":"I F E bB cB dB","257":"D A B C O P eB TB U x iB jB"},F:{"2":"D B C kB lB mB nB U VB PC x","164":"0 1 2 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","1220":"3 4 5 6 7 8 9 BB DB EB CB AB","1348":"T R N"},G:{"16":"SB qB XB sB tB","164":"E uB vB","257":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"16":"OB AC BC CC","164":"G M DC XB EC FC"},J:{"16":"F","164":"A"},K:{"2":"A B C U VB x","164":"Q"},L:{"1220":"M"},M:{"548":"N"},N:{"2":"A B"},O:{"164":"GC"},P:{"164":"G HC IC JC KC LC TB MC"},Q:{"1348":"NC"},R:{"164":"OC"},S:{"548":"fB"}},B:5,C:":matches() CSS pseudo-class"};
