module.exports={A:{A:{"1":"A B","16":"hB","900":"I F E D"},B:{"1":"S UB M HB","1025":"C O P H J K L"},C:{"1":"2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","900":"aB OB gB oB","1025":"0 1 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},D:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"I F E D A B C O P bB cB dB eB TB U x iB jB","16":"V ZB","900":"G SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","16":"D","132":"B C kB lB mB nB U VB PC x"},G:{"1":"qB XB sB tB uB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB","2052":"E vB"},H:{"132":"9B"},I:{"1":"OB G CC DC XB EC FC","16":"AC BC","4097":"M"},J:{"1":"F A"},K:{"132":"A B C U VB x","4100":"Q"},L:{"4097":"M"},M:{"4097":"N"},N:{"1":"A B"},O:{"1":"GC"},P:{"4097":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1025":"fB"}},B:1,C:"maxlength attribute for input and textarea elements"};
