module.exports={A:{A:{"2":"I F E D hB","129":"A B"},B:{"1":"S UB M HB","129":"C O P H J K L"},C:{"2":"aB OB gB oB","129":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"1":"0 1 2 3 4 5 6 7 8 9 d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","16":"G V I F E D A B C O P Y Z a b c","129":"H J K L W X"},E:{"1":"I F E D A B C O P bB cB dB eB TB U x iB jB","16":"G V ZB SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N PC x","2":"D kB lB mB nB","16":"B U VB"},G:{"1":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB qB XB"},H:{"129":"9B"},I:{"1":"M EC FC","16":"AC BC","129":"OB G CC DC XB"},J:{"1":"F","129":"A"},K:{"1":"C","2":"A","16":"B U VB","129":"Q x"},L:{"1":"M"},M:{"129":"N"},N:{"129":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"129":"fB"}},B:1,C:"Search input type"};
