module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"L S UB M HB","2":"C O P H J K"},C:{"1":"AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB","8":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V","8":"I F E","132":"D A B C O P H J K L W X Y Z","260":"a b c d e f g h i"},E:{"1":"P jB","2":"G V I F E D A B C O ZB SB bB cB dB eB TB U x iB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D kB lB mB","8":"B nB","132":"U VB PC","260":"C H J K L x"},G:{"1":"8B","2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B"},H:{"1":"9B"},I:{"1":"M XB EC FC","2":"OB AC BC CC","132":"G DC"},J:{"2":"F A"},K:{"1":"C Q U VB x","2":"A","132":"B"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"8":"fB"}},B:7,C:"WebP image format"};
