module.exports={A:{A:{"132":"I F E D A B hB"},B:{"1":"S UB M HB","132":"C O P H J K L"},C:{"1":"1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","33":"0 K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","132":"aB OB G V I F E D gB oB","292":"A B C O P H J"},D:{"1":"0 1 2 3 4 5 6 7 8 9 z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","132":"G V I F E D A B C O P H J","548":"K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y"},E:{"132":"G V I F E ZB SB bB cB dB","548":"D A B C O P eB TB U x iB jB"},F:{"132":"0 1 2 3 4 5 6 7 8 9 D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB lB mB nB U VB PC x"},G:{"132":"E SB qB XB sB tB uB vB","548":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"16":"9B"},I:{"1":"M","16":"OB G AC BC CC DC XB EC FC"},J:{"16":"F A"},K:{"16":"A B C Q U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"132":"A B"},O:{"16":"GC"},P:{"1":"HC IC JC KC LC TB MC","16":"G"},Q:{"16":"NC"},R:{"16":"OC"},S:{"33":"fB"}},B:4,C:"CSS unicode-bidi property"};
