module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P H J","33":"S UB M HB","129":"K L"},C:{"2":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R gB oB","33":"N IB JB KB LB MB NB FB PB QB RB S"},D:{"16":"G V I F E D A B C O","33":"0 1 2 3 4 5 6 7 8 9 P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G ZB SB","33":"V I F E D A B C O P bB cB dB eB TB U x iB jB"},F:{"2":"D B C kB lB mB nB U VB PC x","33":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"2":"SB qB XB","33":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"16":"AC BC","33":"OB G M CC DC XB EC FC"},J:{"33":"F A"},K:{"2":"A B C U VB x","33":"Q"},L:{"33":"M"},M:{"33":"N"},N:{"2":"A B"},O:{"33":"GC"},P:{"33":"G HC IC JC KC LC TB MC"},Q:{"33":"NC"},R:{"33":"OC"},S:{"2":"fB"}},B:7,C:"CSS line-clamp"};
