module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"P H J K L S UB M HB","2":"C O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G","36":"V I F E D A B C O P H J K L W X Y"},E:{"1":"I F E D A B C O P cB dB eB TB U x iB jB","2":"G V ZB SB bB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C H J K L W X Y Z a b kB lB mB nB U VB PC x"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB","36":"M EC FC"},J:{"1":"A","2":"F"},K:{"2":"A B C U VB x","36":"Q"},L:{"513":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"36":"G","258":"HC IC JC KC LC TB MC"},Q:{"2":"NC"},R:{"258":"OC"},S:{"1":"fB"}},B:1,C:"Web Notifications"};
